/*
 * This file was generated by pso2h.
 */

#ifndef __YV12_H__
#define __YV12_H__

/*
 * This define is the size of the shader in bytes.  More precisely it's the
 * sum of strlen() of every string in the array.
 */
#define YV12_FP_SZ    526

static const char *YV12_fp[] =
{
  "!!ARBfp1.0\n",
  "PARAM c[2] = { { 1, 1.1640625, 0.0625, 2.015625 },\n",
  "{ 0.5, 0.390625, 0.8125, 1.5976562 } };\n",
  "TEMP R0;\n",
  "TEX R0.y, fragment.texcoord[0], texture[0], 2D;\n",
  "ADD R0.x, R0.y, -c[0].z;\n",
  "TEX R0.y, fragment.texcoord[0], texture[1], 2D;\n",
  "ADD R0.z, R0.y, -c[1].x;\n",
  "MUL R0.x, R0, c[0].y;\n",
  "MAD result.color.z, R0, c[0].w, R0.x;\n",
  "TEX R0.y, fragment.texcoord[0], texture[2], 2D;\n",
  "ADD R0.y, R0, -c[1].x;\n",
  "MAD R0.z, -R0, c[1].y, R0.x;\n",
  "MAD result.color.y, -R0, c[1].z, R0.z;\n",
  "MAD result.color.x, R0.y, c[1].w, R0;\n",
  "MOV result.color.w, c[0].x;\n",
  "END\n",
  NULL
};

#endif
