/***************************************************************************
 *  Enums.cs
 *
 *  Copyright (C) 2006-2007 Alan McGovern
 *  Authors:
 *  Alan McGovern (alan.mcgovern@gmail.com)
 ****************************************************************************/

/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW:
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 */

using System;

namespace Mtp
{
    public enum FileType
    {
        WAV,
        MP3,
        WMA,
        OGG,
        AUDIBLE,
        MP4,
        UNDEF_AUDIO,
        WMV,
        AVI,
        MPEG,
        ASF,
        QT,
        UNDEF_VIDEO,
        JPEG,
        JFIF,
        TIFF,
        BMP,
        GIF,
        PICT,
        PNG,
        VCALENDAR1,
        VCALENDAR2,
        VCARD2,
        VCARD3,
        WINDOWSIMAGEFORMAT,
        WINEXEC,
        TEXT,
        HTML,
        FIRMWARE,
        AAC,
        MEDIACARD,
        FLAC,
        MP2,
        M4A,
        DOC,
        XML,
        XLS,
        PPT,
        MHT,
        JP2,
        JPX,
        UNKNOWN
    }
}
