// Generated - Do Not Edit!

internal static class AddinXmlStringCatalog
{
    private static void Strings ()
    {
        // ../src/Dap/Banshee.Dap.AppleDevice/Banshee.Dap.AppleDevice.addin.xml
        Catalog.GetString (@"Apple Devices Support");
        Catalog.GetString (@"Support for most iPod/iPhone/iPad devices.");
        Catalog.GetString (@"Device Support");

        // ../src/Dap/Banshee.Dap.Karma/Banshee.Dap.Karma.addin.xml
        Catalog.GetString (@"Karma Support");
        Catalog.GetString (@"Support for Rio Karma devices.");
        Catalog.GetString (@"Device Support");

        // ../src/Dap/Banshee.Dap.MassStorage/Banshee.Dap.MassStorage.addin.xml
        Catalog.GetString (@"Mass Storage Media Player Support");
        Catalog.GetString (@"Support for USB-drive based devices, including many iAudio devices and Android, Pre, and Galaxy phones.");
        Catalog.GetString (@"Device Support");

        // ../src/Dap/Banshee.Dap.Mtp/Banshee.Dap.Mtp.addin.xml
        Catalog.GetString (@"MTP Media Player Support");
        Catalog.GetString (@"Support for MTP devices, including many Creative Zen devices.");
        Catalog.GetString (@"Device Support");

        // ../src/Extensions/Banshee.AmazonMp3.Store/Banshee.AmazonMp3.Store.addin.xml
        Catalog.GetString (@"Amazon MP3 Store Source");
        Catalog.GetString (@"Browse, search, and purchase music from the Amazon MP3 store.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.AmazonMp3/Banshee.AmazonMp3.addin.xml
        Catalog.GetString (@"Amazon MP3 Import");
        Catalog.GetString (@"Download and import music purchased from Amazon.com.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.Audiobook/Banshee.Audiobook.addin.xml
        Catalog.GetString (@"Audiobooks Library");
        Catalog.GetString (@"Organize audiobooks, lectures, etc.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.BooScript/Banshee.BooScript.addin.xml
        Catalog.GetString (@"Boo Scripting");
        Catalog.GetString (@"Customize and extend Banshee with Boo-language scripts.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.Bpm/Banshee.Bpm.addin.xml
        Catalog.GetString (@"BPM Detection");
        Catalog.GetString (@"Detect the beats per minute (BPM) of your music.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.CoverArt/Banshee.CoverArt.addin.xml
        Catalog.GetString (@"Cover Art Fetching");
        Catalog.GetString (@"Fetch cover art for all items in your library.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.Daap/Banshee.Daap.addin.xml
        Catalog.GetString (@"DAAP Sharing");
        Catalog.GetString (@"Browse and listen to music shared over your local network.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.Emusic.Store/Banshee.Emusic.Store.addin.xml
        Catalog.GetString (@"eMusic Store");
        Catalog.GetString (@"Browse, search, and purchase music and audiobooks from eMusic");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.Emusic/Banshee.Emusic.addin.xml
        Catalog.GetString (@"eMusic Import");
        Catalog.GetString (@"Download and import music purchased from eMusic.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.FileSystemQueue/Banshee.FileSystemQueue.addin.xml
        Catalog.GetString (@"File System Preview Queue");
        Catalog.GetString (@"Preview files without importing to your library.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.Fixup/Banshee.Fixup.addin.xml
        Catalog.GetString (@"Metadata Fixup");
        Catalog.GetString (@"Fix broken and missing metadata using bulk operations.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.InternetArchive/Banshee.InternetArchive.addin.xml
        Catalog.GetString (@"Internet Archive");
        Catalog.GetString (@"Browse and search the Internet Archive's vast media collection.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.InternetRadio/Banshee.InternetRadio.addin.xml
        Catalog.GetString (@"Internet Radio");
        Catalog.GetString (@"Listen to and organize Internet Radio stations.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.Lastfm/Banshee.Lastfm.addin.xml
        Catalog.GetString (@"Last.fm Scrobbling");
        Catalog.GetString (@"Last.fm scrobbling and context pane recommendations.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.LastfmStreaming/Banshee.LastfmStreaming.addin.xml
        Catalog.GetString (@"Last.fm Radio");
        Catalog.GetString (@"Last.fm streaming radio for paid subscribers.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.LibraryWatcher/Banshee.LibraryWatcher.addin.xml
        Catalog.GetString (@"Library Watcher");
        Catalog.GetString (@"Automatically update music and video libraries.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.MiniMode/Banshee.MiniMode.addin.xml
        Catalog.GetString (@"Mini Mode");
        Catalog.GetString (@"Control Banshee through a small window with only playback controls and track information.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.MiroGuide/Banshee.MiroGuide.addin.xml
        Catalog.GetString (@"Miro Guide Podcast Directory");
        Catalog.GetString (@"Browse and subscribe to thousands of audio and video podcasts.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.Mpris/Banshee.Mpris.addin.xml
        Catalog.GetString (@"MPRIS D-Bus interface");
        Catalog.GetString (@"Control Banshee using the MPRIS D-Bus interface.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.MultimediaKeys/Banshee.MultimediaKeys.addin.xml
        Catalog.GetString (@"Multimedia Keys");
        Catalog.GetString (@"Control playback via the multimedia keys on your keyboard.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.NotificationArea/Banshee.NotificationArea.addin.xml
        Catalog.GetString (@"Notification Area Icon");
        Catalog.GetString (@"Display an icon in the notification area for controlling Banshee.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.OpticalDisc/Banshee.OpticalDisc.addin.xml
        Catalog.GetString (@"DVD and Audio CD Support");
        Catalog.GetString (@"Watch DVDs, listen to and rip Audio CDs.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.PlayerMigration/Banshee.PlayerMigration.addin.xml
        Catalog.GetString (@"Importers for Amarok, Rhythmbox and iTunes");
        Catalog.GetString (@"Import your library from Amarok, Rhythmbox or iTunes.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.PlayQueue/Banshee.PlayQueue.addin.xml
        Catalog.GetString (@"Play Queue");
        Catalog.GetString (@"Queue tracks in an ad-hoc playlist, or let the Auto DJ take over.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.Podcasting/Banshee.Podcasting.addin.xml
        Catalog.GetString (@"Podcasts");
        Catalog.GetString (@"Subscribe to podcasts and stream or download episodes.");
        Catalog.GetString (@"Core");

        // ../src/Extensions/Banshee.SoundMenu/Banshee.SoundMenu.addin.xml
        Catalog.GetString (@"Sound Menu Integration");
        Catalog.GetString (@"Control Banshee from the Sound Menu.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.Torrent/Banshee.Torrent.addin.xml
        Catalog.GetString (@"Torrent Downloader");
        Catalog.GetString (@"Support downloading BitTorrent files from podcasts.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.UbuntuOneMusicStore/Banshee.UbuntuOneMusicStore.addin.xml
        Catalog.GetString (@"Ubuntu One Music Store");
        Catalog.GetString (@"This extension grants access to the Ubuntu One Music Store, powered by 7digital.");
        Catalog.GetString (@"Online Sources");

        // ../src/Extensions/Banshee.UPnPClient/Banshee.UPnPClient.addin.xml
        Catalog.GetString (@"UPnP Client");
        Catalog.GetString (@"Browse and play music and videos shared over your local network using UPnP.");
        Catalog.GetString (@"Utilities");

        // ../src/Extensions/Banshee.Wikipedia/Banshee.Wikipedia.addin.xml
        Catalog.GetString (@"Wikipedia");
        Catalog.GetString (@"View artist information from Wikipedia in the context pane.");
        Catalog.GetString (@"Context Pane");

        // ../src/Extensions/Banshee.YouTube/Banshee.YouTube.addin.xml
        Catalog.GetString (@"YouTube");
        Catalog.GetString (@"Show related YouTube videos in the context pane.");
        Catalog.GetString (@"Context Pane");

    }
}
