//
// File.cs
//
// Author:
//   Aaron Bockover <abockover@novell.com>
//
// Copyright (C) 2006-2008 Novell, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using System.IO;

using Hyena;

namespace Banshee.IO.SystemIO
{
    public class File : Banshee.IO.IFile
    {
        public void Delete (SafeUri uri)
        {
            System.IO.File.Delete (uri.LocalPath);
        }

        public bool Exists (SafeUri uri)
        {
            return System.IO.File.Exists (uri.LocalPath);
        }

        public void Move (SafeUri from, SafeUri to)
        {
            System.IO.File.Move (from.LocalPath, to.LocalPath);
        }

        public void Copy (SafeUri from, SafeUri to, bool overwrite)
        {
            System.IO.File.Copy (from.LocalPath, to.LocalPath, overwrite);
        }

        public long GetSize (SafeUri uri)
        {
            try {
                return new System.IO.FileInfo (uri.LocalPath).Length;
            } catch {
                return -1;
            }
        }

        public long GetModifiedTime (SafeUri uri)
        {
            return Hyena.DateTimeUtil.FromDateTime (new System.IO.FileInfo (uri.LocalPath).LastWriteTime);
        }

        public System.IO.Stream OpenRead (SafeUri uri)
        {
            return System.IO.File.OpenRead (uri.LocalPath);
        }

        public System.IO.Stream OpenWrite (SafeUri uri, bool overwrite)
        {
            return overwrite
                ? System.IO.File.Open (uri.LocalPath, FileMode.Create, FileAccess.ReadWrite)
                : System.IO.File.OpenWrite (uri.LocalPath);
        }
    }
}
