// 
// RuntimeTests.cs
// 
// Author:
//   Aaron Bockover <abockover@novell.com>
// 
// Copyright 2010 Novell, Inc.
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

#if ENABLE_TESTS

using System;
using NUnit.Framework;

using JavaScriptCore.Bridge;

namespace JavaScriptCore.Bridge.Tests
{
    [TestFixture]
    public class RuntimeTests
    {
        private JSContext context;

        [TestFixtureSetUp]
        public void Init ()
        {
            context = new JSContext ();
            context.BindManagedRuntime ();
        }

        [Test]
        public void TestExceptionBoundaryJsCatch ()
        {
            context.EvaluateScript ("try { mjs.import = 'foo'; } catch (e) { this.p = e.toString (); }");
            Assert.AreEqual ("IllegalOperationError: Setting properties on this object is not allowed",
                context.GlobalObject.GetProperty ("p").StringValue);
        }

        [Test]
        public void TestExceptionBoundaryRoundTrip ()
        {
            try {
                context.EvaluateScript ("mjs.import = 'foo'");
            } catch (JSException e) {
                Assert.AreEqual ("IllegalOperationError: Setting properties on this object is not allowed",
                    e.Error.StringValue);
            }
        }
    }
}

#endif