//
// DotNetNetworking.cs
//
// Author:
//   Pete Johanson <peter@peterjohanson.com>
//
// Copyright 2010 Pete Johanson
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.NetworkInformation;
using System.Text;

namespace Banshee.Networking
{
    public class DotNetNetworking : INetworkAvailabilityService
    {
        public static bool ManagerPresent { get { try { return Type.GetType ("Mono.Runtime") == null; } catch { return true; } } }

        #region INetworkAvailabilityService Members

        public State State {
            get { return NetworkInterface.GetIsNetworkAvailable () ? State.Connected : State.Disconnected; }
        }

        public event StateChangeHandler StateChange;

        #endregion

        public DotNetNetworking ()
        {
            if (!ManagerPresent) {
                throw new ApplicationException (".NET network detection is only supported on Windows.");
            }

            NetworkChange.NetworkAvailabilityChanged += OnNetworkAvailabilityChanged;
        }

        void OnNetworkAvailabilityChanged (object o, NetworkAvailabilityEventArgs args)
        {
            var handler = StateChange;
            if (handler != null) {
                handler (args.IsAvailable ? State.Connected : State.Disconnected);
            }
        }
    }
}
