/***************************************************************************
 *  TaskAddedEventArgs.cs
 *
 *  Copyright (C) 2007 Michael C. Urbanski
 *  Written by Mike Urbanski <michael.c.urbanski@gmail.com>
 ****************************************************************************/

/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW:
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),
 *  to deal in the Software without restriction, including without limitation
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,
 *  and/or sell copies of the Software, and to permit persons to whom the
 *  Software is furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *  DEALINGS IN THE SOFTWARE.
 */

using System;
using System.Collections.Generic;

using Migo.TaskCore;

namespace Migo.TaskCore.Collections
{
    public class TaskAddedEventArgs<T> : ManipulatedEventArgs<T> where T : Task
    {
        private readonly KeyValuePair<int,T> taskPair;
        private readonly ICollection<KeyValuePair<int,T>> taskPairs;

        public KeyValuePair<int,T> TaskPair
        {
            get { return taskPair; }
        }

        // All indices are listed in ascending order from the start of the
        // list so that in order addition will not affect indices.
        public ICollection<KeyValuePair<int,T>> TaskPairs
        {
            get { return taskPairs; }
        }

        public TaskAddedEventArgs (int pos, T task) : base (task, null)
        {
            if (task == null) {
                throw new ArgumentNullException ("task");
            }

            this.taskPairs = null;
            this.taskPair = new KeyValuePair<int,T> (pos, task);
        }

        public TaskAddedEventArgs (ICollection<KeyValuePair<int,T>> taskPairs)
        {
            if (taskPairs == null) {
                throw new ArgumentNullException ("taskPairs");
            }

            List<T> tsks = new List<T> (taskPairs.Count);

            foreach (KeyValuePair<int,T> kvp in taskPairs) {
                if (kvp.Value == null) {
                    throw new ArgumentNullException (
                        "No task in tasks may be null"
                    );
                }

                tsks.Add (kvp.Value);
            }

            this.Tasks = tsks;
            this.taskPairs = taskPairs;
            this.taskPair = default (KeyValuePair<int,T>);
        }
    }
}
