//
// MusicStoreTests.cs
//
// Author:
//   Rodney Dawes <rodney.dawes@canonical.com>
//
// Copyright (C) 2010 Canonical, Ltd.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

#if ENABLE_TESTS

using Gtk;
using Mono.Addins;
using NUnit.Framework;

namespace Banshee.UbuntuOneMusicStore
{
    [TestFixture]
    public class MusicStoreTests
    {
        [TestFixtureSetUp]
        public void SetUp ()
        {
            Application.Init ();
            AddinManager.Initialize ();
            AddinManager.Registry.Update (null);
        }

        [TestFixtureTearDown]
        public void TearDown ()
        {
            AddinManager.Shutdown ();
        }

        [Test]
        public void TestSource ()
        {
            var source = new UbuntuOneMusicStoreSource ();
            Assert.AreEqual (source.Count, 0);
        }

        [Test]
        public void TestU1MSUrlLoaded ()
        {
            var store = new UbuntuOneMusicStoreSource.StoreWrapper ();
            Assert.IsNotNull (store);
        }

    }
}

#endif
