//
// EventArgs.cs
//
// Author:
//   Alexander Kojevnikov <alexander@kojevnikov.com>
//
// Copyright (C) 2009-2010 Alexander Kojevnikov
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;

namespace Hyena
{
    public class EventArgs<T> : System.EventArgs
    {
        private readonly T value;

        public EventArgs (T value)
        {
            this.value = value;
        }

        public T Value {
            get { return value; }
        }
    }

    public static class EventExtensions
    {
        public static void SafeInvoke<T> (this T @event, params object[] args) where T : class
        {
            var multicast = @event as MulticastDelegate;
            if (multicast != null) {
                foreach (var handler in multicast.GetInvocationList ()) {
                    try {
                        handler.DynamicInvoke (args);
                    } catch (Exception e) {
                        Log.Exception (e);
                    }
                }
            }
        }
    }
}
