//
// SimilarArtistTile.cs
//
// Author:
//   Aaron Bockover <abockover@novell.com>
//
// Copyright (C) 2008 Novell, Inc.
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

using System;
using Mono.Unix;

using Lastfm.Data;
using Banshee.Widgets;

namespace Banshee.Lastfm.Recommendations
{
    public class SimilarArtistTile : Tile
    {
        private SimilarArtist artist;

        public SimilarArtistTile (SimilarArtist artist)
        {
            Artist = artist;
        }

        protected override void OnClicked ()
        {
            if (artist != null) {
                Banshee.Web.Browser.Open (artist.Url);
            }
        }

        public SimilarArtist Artist {
            set {
                artist = value;

                PrimaryText = value.Name;

                try {
                    SecondaryText = String.Format (Catalog.GetString ("{0}% Similarity"), value.MatchAsInt);
                } catch {
                    SecondaryText = Catalog.GetString ("Unknown Similarity");
                }

                try {
                    Pixbuf = new Gdk.Pixbuf (DataCore.GetCachedPathFromUrl (value.SmallImageUrl));
                } catch {
                    Pixbuf = Banshee.Gui.IconThemeUtils.LoadIcon ("generic-artist", 48);
                }
            }
        }
    }
}
